require("love_reactor/all")

Hand = class({
  sprite = love.graphics.newImage("sprites/hand.png"),
  depth = -100,
  angriness = 0,
  init = function(self)
    self.offset = {
      x = self.width / 2,
      y = self.height - 10
    }
  end
}, {drawable, singleton, collidable, singleton})

Jester = class({
  sprite = love.graphics.newImage("sprites/jester.png"),
  scale = {
    x = 2,
    y = 2
  }
}, {drawable, singleton})

Explosion = class({
  depth = -1000,
  sprite = love.graphics.newImage("sprites/explosion.psd"),
  width = 64,
  height = 64,
  animations = {
    splode = {
      frames = {1,2,3},
      speed = .25
    }
  },
  start_animation = "splode",
  init = function(self)
    update_stream.take_one()
      .delay(3)
      .map(self.destroy)
      .map(function()
        love.event.quit()
      end)
  end
}, {animatable})

Sun = class({
  sprite = love.graphics.newImage("sprites/the_sun.png"),
  angriness = 0
}, {drawable, collidable, singleton})

Star = class({
  sprite = love.graphics.newImage("sprites/star.psd"),
  depth = -1000
}, {drawable, movable, collidable})

collision_stream
  .filter(function(e) return e.type == "start" end)
  .map_filter(collision_between(Sun.instances, Hand.instances))
  .map(function(e)
    e.a.angriness = e.a.angriness + 1
    love.audio.newSource(table.pick_random({
      "sfx/slap1.ogg",
      "sfx/slap2.ogg",
      "sfx/slap3.ogg",
    })):play()
    love.audio.newSource(table.pick_random({
      "sfx/blah.wav",
      "sfx/blah2.wav",
      "sfx/blah3.wav",
    })):play()
  end)

update_stream
  .filter(function() return Sun.instance.angriness > 20 end)
  .filter(function() return not Hand.instances.empty() end)
  .map(function()
    Star.new({
      x = love.window.getWidth() / 2,
      y = love.window.getHeight() / 2,
      velocity = vector.mult(vector.normalize(vector.sub(Hand.instance.position, {
        x = love.window.getWidth() / 2,
        y = love.window.getHeight() / 2
      })), 500)
    })
    love.audio.newSource("sfx/schloo.wav"):play()
  end)

collision_stream
  .map_filter(collision_between(Star.instances, Hand.instances))
  .map(function(e)
      Hand.instance.angriness = Hand.instance.angriness + 1
      if Hand.instance.angriness > 50 then
        local explosion = Explosion.new({
          position = Jester.instance.position
        })
        Jester.instances.call('destroy')
        Hand.instances.call('destroy')
        love.audio.newSource("sfx/kabloomers.wav"):play()
      end

      e.a.destroy()
  end)

update_stream
  .map(function()
    for _, sun in pairs(Sun.instances.values()) do
      sun.position = vector.add({x = love.window.getWidth() / 2, y = love.window.getHeight() / 2}, {
        x = math.random(-sun.angriness, sun.angriness),
        y = math.random(-sun.angriness, sun.angriness)
      })
      sun.color = {
        255,
        math.max(255 - sun.angriness * 10, 0),
        math.max(255 - sun.angriness * 10, 0)
      }
      sun.scale = {
        x = math.max(1 * sun.angriness / 2, 1),
        y = math.max(1 * sun.angriness / 2, 1)
      }
    end

    for _, hand in pairs(Hand.instances.values()) do
      hand.color = {
        255,
        math.max(255 - hand.angriness * 10, 0),
        math.max(255 - hand.angriness * 10, 0)
      }
    end
  end)

update_stream
  .filter(function() return not Jester.instances.empty() end)
  .map(function()
    if Jester.instance.position.x ~= love.mouse.getX() and 
       Jester.instance.position.y ~= love.mouse.getY() then
      local delta = vector.sub(Jester.instance.position, {
        x = love.mouse.getX(),
        y = love.mouse.getY()
      })
      local theta = math.atan2(delta.y, delta.x)
      Hand.instance.rotation = theta + (math.pi / 2)
    end

    Jester.instance.position = {
      x = love.mouse.getX(),
      y = love.mouse.getY(),
    }

    Hand.instance.position = {
      x = love.mouse.getX() + 35,
      y = love.mouse.getY() - 20
    }
  end)

function love.load()
  love.graphics.setBackgroundColor(colors.white)
  Hand.new()
  Jester.new()
  Sun.new({
    x = love.window.getWidth() / 2,
    y = love.window.getHeight() / 2
  })

  love.audio.newSource("sfx/song.mp3"):play()
end
